//#include <SPI.h>
#include <U8g2lib.h>      /* LCD */
#include <IRremote.h>     /* For IR remote control  */
#include "Menu.h"         /* Defines our menu */
#include "Thermistor.h"   /* Class for Thermistor */
#include "pin_map.h"      /* Arduino MEGA to RAMPS 1.4 pin mapping */


/*  IR commands for apple remote   */
/*  Last two HEX digits are truncated by dividing raw value by 256 because different Apple remotes give different values in last two digits  */
/*  These can be obtained uding IRremote library IRreceiveDemo sketch  */
#define apple_left 0x77E190   //7856528
#define apple_right 0x77E160  //7856480
#define apple_up 0x77E150     //7856464
#define apple_down 0x77E130   //7856464
#define apple_middle 0x77E1A0 //7856544 
#define apple_menu 0x77E1C0   //7856576
#define apple_repeat 0xFFFFFF // 16777215


bool Running = true;                  /*  true= display temperature, false =display menu */
int units = 0;                        /*  0=Celsius, 1=Fahrenheit, 2 = Kelvin */
uint8_t menu_item_current  = 0;
uint8_t menu_length  = 0;
uint8_t menu_redraw_required = 0;
int16_t last, value;                  /*  store menu selections*/
float Temp_C  = 0;
unsigned int sensorReadFreq = 1;      /*  for reading thermistor Hz*/

Thermistor Thermistor1(TEMP_0_PIN);   /* Create instance of Thermistor class */
U8G2_ST7920_128X64_1_SW_SPI u8g2(U8G2_R0, DOGLCD_SCK , DOGLCD_MOSI, DOGLCD_CS ); /*  Initialize graphics controller */
IRrecv IrReceiver(D44);               /* Create instance of IRrecv class */

// ----------------------------------------------------------------------------


void setup() {
  Serial.begin(115200);
  Serial.println("RAMPS IR REMOTE DIGITAL THERMOMETER");

  //set up Menu
  menu_length = *(&menu_setup + 1) - menu_setup ;
  menu_redraw_required = 1;     // force initial redraw

  last = -1;

  // initialize timers
  noInterrupts();           // disable all interrupts
  // sensor read timer
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;
  OCR1A = round(16000000 / 256 / sensorReadFreq);        // compare match register 16MHz/256/1Hz
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12);    // 256 prescaler
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
  interrupts();             // enable all interrupts

  // IR Remote setup
  Serial.println("Enabling IRin");
  IrReceiver.enableIRIn();  // Start the receiver
  IrReceiver.blink13(true); // Enable feedback LED

  Serial.print(F("Ready to receive IR signals at pin "));
  Serial.println(D44);

  // Menu splash screen
  u8g2.begin(); // Write startup message to LCD
  u8g2.setFontPosTop(); // references position to top of font
  do {
    u8g2.setFont(u8g_font_6x13);
    u8g2.drawStr(10, 10, "Starting...");
  } while ( u8g2.nextPage() );
  delay(500);
}

// ----------------------------------------------------------------------------
// TIMER ISR
// ----------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect)
{
  Temp_C = Thermistor1.getValue();        // read temperature from thermistor
  //Serial.println(Temp_C);
}

// ----------------------------------------------------------------------------


void loop() {
  if (Running) {
    /* Draw temperature to LCD */
    u8g2.firstPage();
    do {
      u8g2.setCursor(10, 15);
      switch (units) {
        case 0:
          u8g2.setCursor(10, 15);
          u8g2.setFont(u8g2_font_fub30_tf);
          u8g2.print(Temp_C , 0);
          u8g2.setFont(u8g2_font_fub17_tf);
          u8g2.print(" C");
          break;
        case 1:
          u8g2.setCursor(10, 15);
          u8g2.setFont(u8g2_font_fub30_tf);
          u8g2.print(Temp_C * 9. / 5. + 32., 0);
          u8g2.setFont(u8g2_font_fub17_tf);
          u8g2.print(" F");
          break;
        case 2:
          u8g2.setFont(u8g2_font_fub30_tf);
          u8g2.print(Temp_C + 273.15, 0);
          u8g2.setFont(u8g2_font_fub17_tf);
          u8g2.print(" K");
          break;
      }
    } while ( u8g2.nextPage() );

  } else {    /* Draw Menu */
    if (menu_redraw_required != 0) {
      u8g2.firstPage();
      do  {
        drawMenu(menu_setup, *(&menu_setup + 1) - menu_setup  );
      } while ( u8g2.nextPage() );
      menu_redraw_required = 0;
    }
  }

  if (IrReceiver.decode()) {
    if (IrReceiver.results.decode_type  == NEC) {
      switch (IrReceiver.results.value / 256) { // need to remove last 2 HEX places because different remotes give different values.
        case apple_left:
          Serial.println("apple_left");
          break;
        case apple_right:
          Serial.println("apple_right");
          break;
        case apple_up:
          Serial.println("apple_up");
          if (Running == 0) {
            value = max(value - 1,0);
            if (value != last) {
              updateMenu(value);                            // update menu bar
              last = value;
            }
          }
          break;
        case apple_down:
          Serial.println("apple_down");
          if (Running == 0) {
             value = min(value + 1,menu_length - 1);
            if (value != last) {
              updateMenu(value);                            // update menu bar
              last = value;
            }
          }
          break;
        case apple_middle:
          Serial.println("apple_middle");
          if (Running == 0) {
            menu_redraw_required = 1;
            menuClick(value);
          }
          break;
        case apple_menu:
          Serial.println("apple_menu");
          if (Running == 0) {
            menu_redraw_required = 1;
            menuClick(value);
          } else {
            Running = 0;
          }
          break;
        case apple_repeat:
          Serial.println("apple_repeat");
          break;
        default:
          Serial.println("default");
          break;
      }

    } else {
      Serial.println("Unknown protocol");
    }
  
    IrReceiver.resume(); // Enable receiving of the next value

  }



}


// ----------------------------------------------------------------------------
// MENU HANDLING ROUTINES
// ----------------------------------------------------------------------------
void drawMenu(const char *menu[], uint8_t menu_len) {
  uint8_t i, h;
  u8g2_uint_t w, d;
  u8g2.setFont(u8g_font_6x13);
  u8g2.setFontRefHeightText();
  h = u8g2.getFontAscent() - u8g2.getFontDescent();
  w = u8g2.getWidth();
  for (  i = 0; i < menu_len; i++ ) {
    d = 10; // menu indent
    if ( i == menu_item_current ) {
      u8g2.drawBox(0, i  * h  , w, h);
      u8g2.setDrawColor(0);
      u8g2.drawStr(d, i * h , menu[i]);
      u8g2.setDrawColor(1);
    } else {
      u8g2.drawStr(d, i * h , menu[i]);
    }
  }
}


void updateMenu(int i) {
  menu_item_current  = i;
  menu_redraw_required = 1;
}



void   menuClick( uint8_t _value) {
  Serial.print("menuClick\t");
  Serial.println(_value);
  units = _value;
  Running = true;
}
